/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package tanks;

import console.Console;
import java.util.Arrays;
import java.util.List;

/**
 *
 * @author bli
 */
public class Player {

    private final String name;
    private final Tank[] tanks;
    private boolean givenUp;

    public Player(String name) {
        this.name = name;
        tanks = new Tank[2];
        tanks[0] = new Tank("Tank 1");
        tanks[1] = new Tank("Tank 2");
    }

    private char readChar(String prompt, Character... accepted) {
        char input;
        List<Character> acceptedList = Arrays.asList(accepted);
        do {
            input = Console.readChar(prompt);
        } while (!acceptedList.contains(input));
        return input;
    }

    private int readInt(String prompt, Integer... accepted) {
        int input;
        List<Integer> acceptedList = Arrays.asList(accepted);
        do {
            input = Console.readInt(prompt);
        } while (!acceptedList.contains(input));
        return input;
    }

    public void move(Player opponent) {
        char input;
        int tankIndex;
        input = readChar("Do you wish to move, pass, or give up? (m/p/g) ", 'm', 'p', 'g');
        if (input == 'g') {
            givenUp = true;
            return;
        }
        if (input == 'p') {
            return;
        }
        tankIndex = readInt("Which tank do you wish to use? (1/2) ", 1, 2);
        Tank tank = tanks[tankIndex - 1];
        if (tank.isDestroyed()) {
            System.out.println("This tank has already been destroyed!");
            return;
        }
        input = readChar("Do you wish to move the tank? (y/n) ", 'y', 'n');
        if (input == 'y') {
            double x = Console.readDouble("Which x coordinate do you wish to move to? ");
            double y = Console.readDouble("Which y coordinate do you wish to move to? ");
            try {
                if (tank.move(new Position(x, y))) {
                    System.out.println("Move was successful!");
                } else {
                    System.out.println("Move was not possible, not enough fuel");
                }
            } catch (PositionException ex) {
                System.out.println("This position is out of the game field!");                
            }
        }
        input = readChar("Do you wish to shoot with the tank? (y/n) ", 'y', 'n');
        if (input == 'y') {
            tankIndex = readInt("Which tank do you wish to shoot? (1/2) ", 1, 2);
            Tank target = opponent.tanks[tankIndex - 1];
            System.out.println("Shot target for " + String.format("%.2f", tank.shoot(target)) + " damage");
            if (target.isDestroyed()) {
                System.out.println("Target destroyed!!!");
            }
        }
    }

    public boolean allDestroyed() {
        for (Tank tank : tanks) {
            if (!tank.isDestroyed()) {
                return false;
            }
        }
        return true;
    }

    public boolean canDamage(Player player) {
        for (Tank tank : tanks) {
            if (tank.isDestroyed() || tank.getAmmo() == 0) {
                continue;
            }
            for (Tank otherTank : player.tanks) {
                if (otherTank.isDestroyed()) {
                    continue;
                }
                double distance = tank.getPosition().distance(otherTank.getPosition());
                if (distance - tank.getFuel() / Tank.fuelConsumption < Tank.shootDistance) {
                    return true;
                }
            }
        }
        return false;
    }

    public String getName() {
        return name;
    }

    public boolean isGivenUp() {
        return givenUp;
    }

    @Override
    public String toString() {
        return "Player{" + "name=" + name + ", tanks=" + Arrays.toString(tanks) + '}';
    }
}
